import { Broker } from './broker';
import { ProjectModel, ProjectRequest } from '../store/projectSlice';

type SaveStateSchema = {
  id: string;
  userId: number;
  name: string;
  dbConnections: any[];
  queries: any[];
  schemas: any[];
  visualizations: any;
  insights: any[];
  project?: {
    id: number | null;
    name: string;
    parent: number | null;
  };
  createdAt: string;
  updatedAt: string;
};

type GetProjectResponse = (data: ProjectModel, status: string) => void;
type GetProjectsResponse = (data: ProjectModel[], status: string) => void;

function transformToProjectModel(saveStates: SaveStateSchema[]): ProjectModel[] {
  return saveStates
    .filter(ss => ss.project && ss.project.id !== null) 
    .map((ss) => ({
      id: String(ss.project!.id),
      name: ss.project!.name,
      parent: ss.project!.parent !== null ? String(ss.project!.parent) : null,
      save_states: [],
      sub_projects: [],
      createdAt: ss.createdAt,
      updatedAt: ss.updatedAt,
    }));
}

export function wsGetProject(userId: string, parentId: string, callback?: GetProjectResponse) {
  Broker.instance().sendMessage(
    {
      key: 'project',
      subKey: 'get',
      body: { user_id: userId, parentId }
    },
    (data: SaveStateSchema, status: string) => {
      const projects = transformToProjectModel([data]);
      if (callback && projects.length > 0) {
        callback(projects[0], status);
      } else if (callback) {
        callback({ 
          id: '', 
          name: '', 
          parent: null, 
          save_states: [], 
          sub_projects: [], 
          createdAt: '', 
          updatedAt: '' 
        }, 'error');
      }
    }
  );
}

export function wsGetProjects(userId: string, parentId: string | null, callback?: GetProjectsResponse) {
  Broker.instance().sendMessage(
    {
      key: 'project',
      subKey: 'getAll',
      body: { parentId }
    },
    (data: SaveStateSchema[], status: string) => {
      const projects = transformToProjectModel(data);
      if (callback) callback(projects, status);
    }
  );
}

export function wsCreateProject(project: ProjectRequest, callback?: GetProjectResponse) {
  Broker.instance().sendMessage(
    {
      key: 'project',
      subKey: 'create',
      body: project
    },
    (data: SaveStateSchema, status: string) => {
      const projects = transformToProjectModel([data]);
      if (callback && projects.length > 0) {
        callback(projects[0], status);
      } else if (callback) {
        callback({ 
          id: '', 
          name: '', 
          parent: null, 
          save_states: [], 
          sub_projects: [], 
          createdAt: '', 
          updatedAt: '' 
        }, 'error');
      }
    }
  );
}

export function wsUpdateProject(id: string, project: ProjectRequest, callback?: GetProjectResponse) {
  Broker.instance().sendMessage(
    {
      key: 'project',
      subKey: 'update',
      body: { id, ...project }
    },
    (data: SaveStateSchema, status: string) => {
      const projects = transformToProjectModel([data]);
      if (callback && projects.length > 0) {
        callback(projects[0], status);
      } else if (callback) {
        callback({ 
          id: '', 
          name: '', 
          parent: null, 
          save_states: [], 
          sub_projects: [], 
          createdAt: '', 
          updatedAt: '' 
        }, 'error');
      }
    }
  );
}

export function wsDeleteProject(id: string, callback?: (data: { success: boolean }, status: string) => void) {
  Broker.instance().sendMessage(
    {
      key: 'project',
      subKey: 'delete',
      body: { id }
    },
    callback
  );
}

export function wsProjectSubscription(callback: (data: ProjectModel, status: string) => void) {
  const id = Broker.instance().subscribe((data: SaveStateSchema, status: string) => {
    const projects = transformToProjectModel([data]);
    if (projects.length > 0) {
      callback(projects[0], status);
    }
  }, 'project');
  return () => {
    Broker.instance().unSubscribe('project', id);
  };
}
