import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import { RootState } from './store';

export type ProjectRequest = {
  name: string;
  parent?: string | null;
  userId: string;
};

export type ProjectModel = {
  id: string;
  name: string;
  parent?: string | null;
  save_states?: any[];
  sub_projects?: ProjectModel[];
  createdAt: string;
  updatedAt: string;
};

export interface ProjectState {
  projects: { [id: string]: ProjectModel };
  currentPath: string[];
  currentProject: string | null;
  isLoading: boolean;
}

const initialState: ProjectState = {
  projects: {},
  currentPath: [],
  currentProject: null,
  isLoading: false
};

export const projectSlice = createSlice({
  name: 'project',
  initialState,
  reducers: {
    setProjects: (state, action: PayloadAction<ProjectModel[]>) => {
      state.projects = {};
      action.payload.forEach(project => {
        state.projects[project.id] = project;
      });
    },
    addProject: (state, action: PayloadAction<ProjectModel>) => {
      state.projects[action.payload.id] = action.payload;
    },
    updateProject: (state, action: PayloadAction<ProjectModel>) => {
      state.projects[action.payload.id] = action.payload;
    },
    deleteProject: (state, action: PayloadAction<string>) => {
      delete state.projects[action.payload];
    },
    setCurrentProject: (state, action: PayloadAction<string | null>) => {
      state.currentProject = action.payload;
    },
    addToPath: (state, action: PayloadAction<string>) => {
      state.currentPath.push(action.payload);
    },
    setPath: (state, action: PayloadAction<string[]>) => {
      state.currentPath = action.payload;
    },
    goBackInPath: (state) => {
      state.currentPath.pop();
    },
    setLoading: (state, action: PayloadAction<boolean>) => {
      state.isLoading = action.payload;
    },
  },
});

export const {
  setProjects,
  addProject,
  updateProject,
  deleteProject,
  setCurrentProject,
  addToPath,
  setPath,
  goBackInPath,
  setLoading,
} = projectSlice.actions;

export const selectProjects = (state: RootState) => state.project.projects;
export const selectCurrentProject = (state: RootState) => state.project.currentProject;
export const selectCurrentPath = (state: RootState) => state.project.currentPath;
export const selectIsLoading = (state: RootState) => state.project.isLoading;

export default projectSlice.reducer;