import React, { useEffect, useMemo, useState } from 'react';
import {
  Button,
  Dialog,
  DialogContent,
  DropdownItem,
  Icon,
  Input,
  nilUUID,
  SaveStateI,
  useAppDispatch,
  useAppSelector,
  useAuthCache,
  useAuthentication,
  useSessionCache,
  wsDeleteState,
} from '../..';
import { deleteSaveState, selectSaveState } from '../../data-access/store/sessionSlice';
import { clearQB } from '../../data-access/store/querybuilderSlice';
import { clearSchema } from '../../data-access/store/schemaSlice';
import { Popover, PopoverContent, PopoverTrigger } from '../../components/layout/Popover';
import { useHandleDatabase } from './useHandleDatabase';
import { addProject, selectCurrentPath, selectProjects, setCurrentProject, setPath, setProjects, deleteProject, addToPath } from '../../data-access/store/projectSlice';
import { Breadcrumb } from '../../components/breadcrumb/Breadcrumb';
import { wsCreateProject, wsDeleteProject, wsGetProjects } from '@graphpolaris/shared/lib/data-access/broker/wsProject';

type Props = {
  onClose: () => void;
  saveStates: { [id: string]: SaveStateI };
  changeActive: (val: 'add' | 'update') => void;
  setSelectedSaveState: (val: SaveStateI) => void;
};

type DatabaseTableHeaderTypes = 'name' | 'protocol' | 'url';

const DataTableHeader = ({
  name,
  orderBy,
  setOrderBy,
}: {
  name: DatabaseTableHeaderTypes;
  orderBy: [DatabaseTableHeaderTypes, 'asc' | 'desc'];
  setOrderBy: (val: [DatabaseTableHeaderTypes, 'asc' | 'desc']) => void;
}) => {
  return (
    <th
      scope="col"
      className="text-left hover:underline cursor-pointer"
      onClick={() => setOrderBy([name, orderBy[1] === 'desc' && orderBy[0] === name ? 'asc' : 'desc'])}
    >
      <span className="flex items-center gap-0">
        {name}
        {orderBy[0] !== name && <div className="min-w-6 min-h-4 " />}
        {orderBy[0] === name && orderBy[1] === 'asc' && <Icon component="icon-[ic--baseline-arrow-drop-up]" />}
        {orderBy[0] === name && orderBy[1] === 'desc' && <Icon component="icon-[ic--baseline-arrow-drop-down]" />}
      </span>
    </th>
  );
};

export function Databases({ onClose, saveStates, changeActive, setSelectedSaveState }: Props) {
  const dispatch = useAppDispatch();
  const auth = useAuthentication();
  const authCache = useAuthCache();
  const session = useSessionCache();
  const databaseHandler = useHandleDatabase();
  const [orderBy, setOrderBy] = useState<[DatabaseTableHeaderTypes, 'asc' | 'desc']>(['name', 'desc']);
  const [sharing, setSharing] = useState<boolean>(false);
  const [showNewProjectDialog, setShowNewProjectDialog] = useState(false);
  const [newProjectName, setNewProjectName] = useState('');
  const currentPath = useAppSelector(selectCurrentPath);
  const projects = useAppSelector(selectProjects);

  useEffect(() => {
    if (currentPath.length > 0 && currentPath[currentPath.length - 1]) {
      wsGetProjects(authCache.authentication?.userID || '', currentPath[currentPath.length - 1], (projects) => {
        dispatch(setProjects(projects));
      });
    } else {
      wsGetProjects(authCache.authentication?.userID || '', null, (projects) => {
        dispatch(setProjects(projects));
      });
    }
  }, [currentPath]);

  const orderedProjects = useMemo(() =>
    Object.values(projects)
      .filter(project => project.parent === (currentPath.length > 0 ? currentPath[currentPath.length - 1] : null))
      .sort((a, b) => {
        if (orderBy[1] === 'asc') {
          return a.name.toLowerCase().localeCompare(b.name.toLowerCase());
        }
        return b.name.toLowerCase().localeCompare(a.name.toLowerCase());
      }),
    [projects, currentPath, orderBy]
  );

  const orderedSaveStates = useMemo(
    () =>
      Object.keys(saveStates)
        .filter(key => {
          const saveState = saveStates[key];
          const currentProjectId = currentPath[currentPath.length - 1] || null;
          
          // only looking at actual db entries
          if (!saveState.dbConnections?.length) {
            return false;
          }

          // show dbs withh no projects
          if (!currentProjectId) {
            return !saveState.project;
          }
          
          // show dbs of the project
          return String(saveState.project?.id) === String(currentProjectId);
        })
        .sort((a, b) => {
          const dir = orderBy[1] === 'asc' ? 1 : -1;
          if (orderBy[0] === 'name') {
            if (saveStates[a].name.toLowerCase() <= saveStates[b].name.toLowerCase()) return dir;
            else return -dir;
          } else {
            if (
              saveStates[a].dbConnections?.[0]?.[orderBy[0]].toLowerCase() <=
              saveStates[b].dbConnections?.[0]?.[orderBy[0]].toLowerCase()
            )
              return dir;
            else return -dir;
          }
        }),
    [saveStates, orderBy, currentPath],
  );

  const handleCreateProject = () => {
    if (newProjectName) {
      wsCreateProject(
        {
          name: newProjectName,
          parent: currentPath.length > 0 ? currentPath[currentPath.length - 1] : null,
          userId: authCache.authentication?.userID || ''
        },
        (data, status) => {
          if (status === 'success' && data) {
            dispatch(addProject(data));
            setShowNewProjectDialog(false);
            setNewProjectName('');
            
            wsGetProjects(authCache.authentication?.userID || '', currentPath.length > 0 ? currentPath[currentPath.length - 1] : null, (projects) => {
              dispatch(setProjects(projects));
            });
          }
        }
      );
    }
  };

  return (
    <div className="flex flex-col gap-4">
      <div className="flex gap-x-4">
        <Button
          label="Database"
          variantType="primary"
          iconComponent="icon-[ic--baseline-add]"
          iconPosition="leading"
          onClick={() => changeActive('add')}
        />
        <Button
          label="Project"
          variantType="primary"
          iconComponent="icon-[mdi--folder-plus]"
          iconPosition="leading"
          onClick={() => setShowNewProjectDialog(true)}
        />
        <Button
          label="Learn how to use"
          variant="ghost"
          iconComponent="icon-[mdi--play]"
          iconPosition="leading"
          onClick={() => window.open('https://graphpolaris.com/', '_blank')}
        />
      </div>
      <Breadcrumb
        paths={currentPath}
        onNavigate={(index) => {
          const newPath = currentPath.slice(0, index + 1);
          dispatch(setPath(newPath));
          dispatch(setCurrentProject(newPath[newPath.length - 1] || null));
        }}
      />

      <Dialog open={showNewProjectDialog} onOpenChange={(open) => !open && setShowNewProjectDialog(false)}>
        <DialogContent>
          <div className="p-4">
            <h2 className="text-lg font-semibold mb-4">Create New Project</h2>
            <Input
              type="text"
              label="Project Name"
              value={newProjectName}
              onChange={setNewProjectName}
              placeholder="Enter project name"
            />
            <div className="flex justify-end gap-2 mt-4">
              <Button
                variant="outline"
                label="Cancel"
                onClick={() => setShowNewProjectDialog(false)}
              />
              <Button
                variantType="primary"
                label="Create"
                onClick={handleCreateProject}
                disabled={!newProjectName}
              />
            </div>
          </div>
        </DialogContent>
      </Dialog>

      <table className="w-full">
        <thead>
          <tr>
            <th></th>
            <DataTableHeader name="name" orderBy={orderBy} setOrderBy={setOrderBy} />
            <DataTableHeader name="protocol" orderBy={orderBy} setOrderBy={setOrderBy} />
            <DataTableHeader name="url" orderBy={orderBy} setOrderBy={setOrderBy} />
            <th></th>
          </tr>
        </thead>
        <tbody className="divide-y divide-white overflow-auto table-fixed">{orderedProjects.map((project) => (
            <tr key={project.id}>
              <td className="w-8">
                <Icon component="icon-[mdi--folder]" size={20} className="text-black-500" />
              </td>
              <td>
                <Button
                  variant="ghost"
                  onClick={() => {
                    dispatch(addToPath(project.id));
                    dispatch(setCurrentProject(project.id));
                  }}
                >
                  {project.name}
                </Button>
              </td>
              <td></td>
              <td></td>
              <td className="text-right flex justify-end">
                <Button
                  iconComponent="icon-[mdi--trash-outline]"
                  variant="ghost"
                  onClick={() => {
                    wsDeleteProject(project.id);
                    dispatch(deleteProject(project.id));
                  }}
                />
              </td>
            </tr>
          ))}

          {orderedSaveStates.map((key) => (
            <tr key={key}>
              <td className="w-8">
                <Icon component="icon-[mdi--database-outline]" size={20} />
              </td>
              <td>
                <Button
                  variant={session.currentSaveState === key ? 'outline' : 'ghost'}
                  onClick={() => {
                    if (key !== session.currentSaveState) {
                      dispatch(selectSaveState(key));
                      dispatch(clearQB());
                      dispatch(clearSchema());
                      onClose();
                    }
                  }}
                >
                  {saveStates[key].name}
                </Button>
              </td>
              <td className="text-left">
                <span className="font-light">{saveStates[key].dbConnections?.[0]?.protocol}</span>
              </td>
              <td className="text-left">
                <span className="font-light">{saveStates[key].dbConnections?.[0]?.url}</span>
              </td>
              <td className="text-right flex justify-end">
                <Button
                  iconComponent="icon-[mdi--trash-outline]"
                  variant="ghost"
                  onClick={() => {
                    if (session.currentSaveState === key) {
                      dispatch(clearQB());
                      dispatch(clearSchema());
                    }
                    wsDeleteState(key);
                    dispatch(deleteSaveState(key));
                  }}
                />
                <Popover>
                  <PopoverTrigger>
                    <Button iconComponent="icon-[mi--options-vertical]" variant="ghost" />
                  </PopoverTrigger>
                  <PopoverContent className="w-56 z-50 bg-light rounded-sm border-[1px] outline-none">
                    <DropdownItem
                      value="Open"
                      onClick={() => {
                        if (key !== session.currentSaveState) {
                          dispatch(selectSaveState(key));
                          dispatch(clearQB());
                          dispatch(clearSchema());
                        }
                        onClose();
                      }}
                    />
                    <DropdownItem
                      value="Update"
                      onClick={() => {
                        changeActive('update');
                        setSelectedSaveState(saveStates[key]);
                      }}
                    />
                    <DropdownItem
                      value="Clone"
                      onClick={() => {
                        databaseHandler.submitDatabaseChange(
                          { ...saveStates[key], name: saveStates[key].name + ' (copy)', id: nilUUID },
                          'add',
                          true,
                          () => {},
                        );
                        setSelectedSaveState(saveStates[key]);
                      }}
                    />
                    <DropdownItem
                      value={sharing ? 'Creating Share Link' : 'Share'}
                      disabled={sharing}
                      onClick={async () => {
                        setSharing(true);
                        await auth.shareLink();
                        setSharing(false);
                      }}
                    />
                    <DropdownItem
                      value="Delete"
                      onClick={() => {
                        if (session.currentSaveState === key) {
                          dispatch(clearQB());
                          dispatch(clearSchema());
                        }
                        wsDeleteState(key);
                        dispatch(deleteSaveState(key));
                      }}
                      className="text-danger"
                    />
                  </PopoverContent>
                </Popover>
              </td>
            </tr>
          ))}
        </tbody>
      </table>
    </div>
  );
}
